@@ -1,13 +1,13 @@
 #!/usr/bin/env python3
 '''__main__.py'''
 
-# Internal python libraries
+# Internal Libraries
 import os
 import sys
 import tempfile
 from shutil import rmtree
 
-version = '21w05b'
+version = '21w10a'
 
 def file_type(file: str) -> str:
     if(not os.path.isfile(file)):
@@ -38,8 +38,14 @@ def frame_type(inp: str):
         return inp[:-4]
     return int(inp)
 
+def comma_type(inp: str) -> list:
+    from usefulFunctions import cleanList
+    ms = inp.split(',')
+    ms = cleanList(ms, '\r\n\t')
+    return ms
+
 def add_argument(*names, nargs=1, type=str, default=None, action='default',
-    range=None, choices=None, group=None, help='', extra=''):
+    range=None, choices=None, group=None, stack=None, help='', extra=''):
     newDic = {}
     newDic['names'] = names
     newDic['nargs'] = nargs
@@ -51,11 +57,11 @@ def add_argument(*names, nargs=1, type=str, default=None, action='default',
     newDic['range'] = range
     newDic['choices'] = choices
     newDic['grouping'] = group
+    newDic['stack'] = stack
     return [newDic]
 
 def main_options():
     ops = []
-
     ops += add_argument('urlOps', nargs=0, action='grouping')
     ops += add_argument('--format', type=str, group='urlOps',
         default='bestvideo[ext=mp4]+bestaudio[ext=m4a]/mp4',
@@ -104,9 +110,6 @@ def main_options():
     ops += add_argument('--constant_rate_factor', '-crf', type=int, default=15,
         group='exportMediaOps', range='0 to 51',
         help='set the quality for video using the crf method.')
-    ops += add_argument('--render', default='auto',
-        choices=['av', 'opencv', 'auto'],
-        help='choice which rendering method to use. (experimental)')
 
     ops += add_argument('motionOps', nargs=0, action='grouping')
     ops += add_argument('--dilates', '-d', type=int, default=2, range='0 to 5',
@@ -128,7 +131,27 @@ def main_options():
     ops += add_argument('--export_to_final_cut_pro', '-exf', action='store_true',
         help='export as an XML file for Final Cut Pro instead of outputting a media file.')
     ops += add_argument('--export_as_json', action='store_true',
-        help='export as a JSON file that can be read by auto-editor later. (experimental)')
+        help='export as a JSON file that can be read by auto-editor later.')
+
+    ops += add_argument('--render', default='auto', choices=['av', 'opencv', 'auto'],
+        help='choice which method to render video.')
+    ops += add_argument('--scale', type=float_type, default=1,
+        help='scale the output media file by a certian size (change resolution by a factor)',
+        extra='Hello')
+
+    ops += add_argument('--zoom', type=comma_type, nargs='*',
+        help='set when and how a zoom will occur.',
+        extra='The arguments are: start,end,start_zoom,end_zoom,x,y,inter,hold' \
+            '\nThere must be at least 4 comma args. x and y default to centerX and centerY' \
+            '\nThe default interpolation is linear.')
+    ops += add_argument('--rectangle', type=comma_type, nargs='*',
+        help='overlay a rectangle shape on the video.',
+        extra='The arguments are: start,end,x1,y1,x2,y2,color,thickness' \
+            '\nThere must be at least 6 comma args. The rectangle is solid if' \
+            ' thickness is not defined.\n The default color is #000.')
+
+    ops += add_argument('--background', type=str, default='#000',
+        help='set the color of the background when the video is moved.')
 
     ops += add_argument('--ignore', nargs='*',
         help='the range that will be marked as "loud"')
@@ -164,6 +187,10 @@ def main_options():
     ops += add_argument('--quiet', '-q', action='store_true',
         help='display less output.')
 
+    ops += add_argument('--combine_files', action='store_true',
+        help='combine all input files into one before editing.')
+    ops += add_argument('--preview', action='store_true',
+        help='show stats on how the input will be cut.')
     ops += add_argument('--no_open', action='store_true',
         help='do not open the file after editing is done.')
     ops += add_argument('--min_clip_length', '-mclip', type=frame_type, default=3,
@@ -172,55 +199,52 @@ def main_options():
     ops += add_argument('--min_cut_length', '-mcut', type=frame_type, default=6,
         range='0 to Infinity',
         help="set the minimum length a cut can be. If a cut is too short, don't cut")
-    ops += add_argument('--combine_files', action='store_true',
-        help='combine all input files into one before editing.')
-    ops += add_argument('--preview', action='store_true',
-        help='show stats on how the input will be cut.')
 
-    ops += add_argument('--frame_margin', '-m', type=frame_type, default=6,
-        range='0 to Infinity',
-        help='set how many "silent" frames of on either side of "loud" sections be included.')
+    ops += add_argument('--output_file', '--output', '-o', nargs='*',
+        help='set the name(s) of the new output.')
     ops += add_argument('--silent_threshold', '-t', type=float_type, default=0.04,
         range='0 to 1',
         help='set the volume that frames audio needs to surpass to be "loud".')
-    ops += add_argument('--video_speed', '--sounded_speed', '-v', type=float_type,
-        default=1.00, range='0 to 999999',
-        help='set the speed that "loud" sections should be played at.')
     ops += add_argument('--silent_speed', '-s', type=float_type, default=99999,
         range='0 to 99999',
         help='set the speed that "silent" sections should be played at.')
-    ops += add_argument('--output_file', '--output', '-o', nargs='*',
-        help='set the name(s) of the new output.')
+    ops += add_argument('--video_speed', '--sounded_speed', '-v', type=float_type,
+        default=1.00, range='0 to 999999',
+        help='set the speed that "loud" sections should be played at.')
+    ops += add_argument('--frame_margin', '-m', type=frame_type, default=6,
+        range='0 to Infinity',
+        help='set how many "silent" frames of on either side of "loud" sections be included.')
 
     ops += add_argument('--help', '-h', action='store_true',
         help='print info about the program or an option and exit.')
     ops += add_argument('(input)', nargs='*',
         help='the path to a file, folder, or url you want edited.')
-
     return ops
 
-def generate_options():
+def create_options():
     ops = []
-    ops += add_argument('--fps', type=float, default=30.0,
+    ops += add_argument('--frame_rate', '-fps', '-r', type=float, default=30.0,
         help='set the framerate for the output video.')
-    ops += add_argument('--duration', type=int, default=10,
+    ops += add_argument('--duration', '-d', type=int, default=10,
         help='set the length of the video (in seconds).')
-    ops += add_argument('--width', type=int, default=640,
+    ops += add_argument('--width', type=int, default=1280, #640
         help='set the pixel width of the video.')
-    ops += add_argument('--height', type=int, default=360,
+    ops += add_argument('--height', type=int, default=720, #360
         help='set the pixel height of the video.')
     ops += add_argument('--output_file', '--output', '-o', type=str,
         default='testsrc.mp4')
     ops += add_argument('--my_ffmpeg', action='store_true',
         help='use your ffmpeg and other binaries instead of the ones packaged.')
-
     ops += add_argument('--help', '-h', action='store_true',
         help='print info about the program or an option and exit.')
+    ops += add_argument('(input)', nargs='*',
+        help='the template')
     return ops
 
 def info_options():
     ops = []
-
+    ops += add_argument('--fast', action='store_true',
+        help='skip information that is very slow to get.')
     ops += add_argument('--my_ffmpeg', action='store_true',
         help='use your ffmpeg and other binaries instead of the ones packaged.')
     ops += add_argument('--help', '-h', action='store_true',
@@ -229,13 +253,6 @@ def info_options():
         help='the path to a file you want inspected.')
     return ops
 
-def genHelp(option_data):
-    for option in option_data:
-        if(option['grouping'] is None):
-            print(' ', ', '.join(option['names']) + ':', option['help'])
-            if(option['action'] == 'grouping'):
-                print('     ...')
-    print('')
 
 def main():
     dirPath = os.path.dirname(os.path.realpath(__file__))
@@ -247,9 +264,7 @@ def main():
         print(f'Auto-Editor version {version}\nPlease use --version instead.')
         sys.exit()
 
-    # If the users just runs: $ auto-editor
     if(sys.argv[1:] == []):
-        # Print
         print('\nAuto-Editor is an automatic video/audio creator and editor.\n')
         print('By default, it will detect silence and create a new video with ')
         print('those sections cut out. By changing some of the options, you can')
@@ -263,44 +278,34 @@ def main():
     from vanparse import ParseOptions
     from usefulFunctions import Log, Timer
 
-    if(len(sys.argv) > 1 and sys.argv[1] == 'generate_test'):
-        option_data = generate_options()
-        args = ParseOptions(sys.argv[2:], Log(), 'generate_test', option_data)
+    subcommands = ['create', 'test', 'info']
 
-        if(args.help):
-            genHelp(option_data)
-            sys.exit()
+    if(len(sys.argv) > 1 and sys.argv[1] in subcommands):
+        if(sys.argv[1] == 'create'):
+            args = ParseOptions(sys.argv[2:], Log(), 'create', create_options())
 
-        from generateTestMedia import generateTestMedia
-        from usefulFunctions import FFmpeg
+            from create import create
+            from usefulFunctions import FFmpeg
 
-        ffmpeg = FFmpeg(dirPath, args.my_ffmpeg, Log())
-        generateTestMedia(ffmpeg, args.output_file, args.fps, args.duration,
-            args.width, args.height)
-        sys.exit()
+            ffmpeg = FFmpeg(dirPath, args.my_ffmpeg, True, Log())
+            create(ffmpeg, args.input, args.output_file, args.frame_rate, args.duration,
+                args.width, args.height, Log())
 
-    elif(len(sys.argv) > 1 and sys.argv[1] == 'test'):
+        if(sys.argv[1] == 'test'):
+            from testAutoEditor import testAutoEditor
+            testAutoEditor()
 
-        from testAutoEditor import testAutoEditor
-        testAutoEditor()
-        sys.exit()
+        if(sys.argv[1] == 'info'):
+            args = ParseOptions(sys.argv[2:], Log(), 'info', info_options())
 
-    elif(len(sys.argv) > 1 and sys.argv[1] == 'info'):
-        option_data = info_options()
-        args = ParseOptions(sys.argv[2:], Log(), 'info', option_data)
+            from info import getInfo
+            from usefulFunctions import FFmpeg, FFprobe
 
-        if(args.help):
-            genHelp(option_data)
-            sys.exit()
+            log = Log()
+            ffmpeg = FFmpeg(dirPath, args.my_ffmpeg, False, log)
+            ffprobe = FFprobe(dirPath, args.my_ffmpeg, False, log)
 
-        from info import getInfo
-        from usefulFunctions import FFmpeg, FFprobe
-
-        log = Log()
-        ffmpeg = FFmpeg(dirPath, args.my_ffmpeg, log)
-        ffprobe = FFprobe(dirPath, args.my_ffmpeg, log)
-
-        getInfo(args.input, ffmpeg, ffprobe, log)
+            getInfo(args.input, ffmpeg, ffprobe, args.fast, log)
         sys.exit()
     else:
         option_data = main_options()
@@ -308,26 +313,16 @@ def main():
 
     timer = Timer(args.quiet)
 
-    # Print the help screen for the entire program.
-    if(args.help):
-        print('\n  Have an issue? Make an issue. '\
-            'Visit https://github.com/wyattblue/auto-editor/issues\n')
-        print('  The help option can also be used on a specific option:')
-        print('      auto-editor --frame_margin --help\n')
-        genHelp(option_data)
-        sys.exit()
-
     del option_data
 
     from usefulFunctions import FFmpeg, FFprobe, sep
-    ffmpeg = FFmpeg(dirPath, args.my_ffmpeg, Log())
-    ffprobe = FFprobe(dirPath, args.my_ffmpeg, Log())
+    ffmpeg = FFmpeg(dirPath, args.my_ffmpeg, args.show_ffmpeg_debug, Log())
+    ffprobe = FFprobe(dirPath, args.my_ffmpeg, args.show_ffmpeg_debug, Log())
 
     makingDataFile = (args.export_to_premiere or args.export_to_resolve or
         args.export_to_final_cut_pro or args.export_as_json)
     is64bit = '64-bit' if sys.maxsize > 2**32 else '32-bit'
 
-
     if(args.debug and args.input == []):
         import platform
 
@@ -342,6 +337,10 @@ def main():
         print('Auto-Editor version', version)
         sys.exit()
 
+    TEMP = tempfile.mkdtemp()
+    log = Log(args.debug, args.quiet, temp=TEMP)
+    log.debug(f'\n   - Temp Directory: {TEMP}')
+
     if(is64bit == '32-bit'):
         log.warning('You have the 32-bit version of Python, which may lead to' \
             'memory crashes.')
@@ -350,17 +349,12 @@ def main():
         print('Auto-Editor version', version)
         sys.exit()
 
-    TEMP = tempfile.mkdtemp()
-    log = Log(args.debug, args.show_ffmpeg_debug, args.quiet, temp=TEMP)
-    log.debug(f'\n   - Temp Directory: {TEMP}')
-
     ffmpeg.updateLog(log)
     ffprobe.updateLog(log)
 
-    from wavfile import read
     from usefulFunctions import isLatestVersion
 
-    if(not args.quiet and isLatestVersion(version, log)):
+    if(not args.quiet and not isLatestVersion(version, log)):
         log.print('\nAuto-Editor is out of date. Run:\n')
         log.print('    pip3 install -U auto-editor')
         log.print('\nto upgrade to the latest version.\n')
@@ -404,12 +398,16 @@ def main():
     speeds = [args.silent_speed, args.video_speed]
     log.debug(f'   - Speeds: {speeds}')
 
+    from wavfile import read
     audioExtensions = ['.wav', '.mp3', '.m4a', '.aiff', '.flac', '.ogg', '.oga',
         '.acc', '.nfa', '.mka']
-
-    # videoExtensions = ['.mp4', '.mkv', '.mov', '.webm', '.ogv']
+    sampleRate = None
 
     for i, INPUT_FILE in enumerate(inputList):
+
+        if(len(inputList) > 1):
+            log.conwrite(f'Working on {INPUT_FILE}')
+
         fileFormat = INPUT_FILE[INPUT_FILE.rfind('.'):]
 
         chunks = None
@@ -436,7 +434,7 @@ def main():
             sampleRate = ffprobe.getSampleRate(INPUT_FILE)
             if(sampleRate == 'N/A'):
                 sampleRate = '48000'
-                log.warning(f"Samplerate couldn't be detected, using {sampleRate}.")
+                log.warning(f"Samplerate wasn't detected, so it will be set to {sampleRate}.")
         else:
             sampleRate = str(args.sample_rate)
         log.debug(f'   - sampleRate: {sampleRate}')
@@ -446,7 +444,7 @@ def main():
                 # audio bitrate not supported in the mkv container.
                 audioBitrate = None
             else:
-                audioBitrate = ffprobe.getPrettyABitrate(INPUT_FILE)
+                audioBitrate = ffprobe.getPrettyBitrate(INPUT_FILE, 'a')
                 if(audioBitrate == 'N/A'):
                     log.warning("Couldn't automatically detect audio bitrate.")
                     audioBitrate = None
@@ -455,6 +453,7 @@ def main():
 
         log.debug(f'   - audioBitrate: {audioBitrate}')
 
+        audioData = None
         audioFile = fileFormat in audioExtensions
         if(audioFile):
             if(args.force_fps_to is None):
@@ -482,7 +481,10 @@ def main():
                 fps = int(ffprobe.getFrameRate(INPUT_FILE))
             else:
                 fps = ffprobe.getFrameRate(INPUT_FILE)
-            log.debug(f'Frame rate: {fps}')
+
+            if(fps < 1):
+                log.error(f"Frame rate cannot be below 1. fps: {fps}" \
+                    f"\nInput file: {INPUT_FILE}")
 
             tracks = args.force_tracks_to
             if(tracks is None):
@@ -559,9 +561,16 @@ def main():
             hasLoud = combineArrs(audioList, motionList, args.edit_based_on, log)
             del audioList, motionList
 
+            effects = []
+            if(args.zoom != []):
+                from cutting import applyZooms
+                effects += applyZooms(args.zoom, audioData, sampleRate, fps, log)
+            if(args.rectangle != []):
+                from cutting import applyRects
+                effects += applyRects(args.rectangle, audioData, sampleRate, fps, log)
+
             chunks = applySpacingRules(hasLoud, fps, args.frame_margin,
-                args.min_clip_length, args.min_cut_length, args.ignore,
-                args.cut_out, log)
+                args.min_clip_length, args.min_cut_length, args.ignore, args.cut_out, log)
             del hasLoud
 
         clips = []
@@ -591,10 +600,11 @@ def main():
             preview(INPUT_FILE, chunks, speeds, fps, audioFile, log)
             continue
 
-        if(args.export_to_premiere or args.export_to_resolve):
+        if(args.export_to_premiere or args.export_to_resolve or
+            args.export_to_final_cut_pro):
             from editor import editorXML
-            editorXML(INPUT_FILE, TEMP, newOutput, clips, chunks, tracks, sampleRate,
-                audioFile, args.export_to_resolve, fps, log)
+            editorXML(INPUT_FILE, TEMP, newOutput, ffprobe, clips, chunks, tracks,
+                sampleRate, audioFile, args.export_to_resolve, fps, log)
             continue
 
         if(audioFile):
@@ -612,22 +622,33 @@ def main():
         continueVid = handleAudioTracks(ffmpeg, newOutput, args, tracks, chunks, speeds,
             fps, TEMP, log)
         if(continueVid):
-
             if(args.render == 'auto'):
-                try:
-                    import av
-                    args.render = 'av'
-                except ImportError:
+                if(args.zoom != [] or args.rectangle != []):
                     args.render = 'opencv'
+                else:
+                    try:
+                        import av
+                        args.render = 'av'
+                    except ImportError:
+                        args.render = 'opencv'
 
             log.debug(f'Using {args.render} method')
             if(args.render == 'av'):
+
+                if(args.zoom != []):
+                    log.error('Zoom effect is not supported on the av render method.')
+
+                if(args.rectangle != []):
+                    log.error('Rectangle effect is not supported on the av render method.')
+
                 from renderVideo import renderAv
-                renderAv(ffmpeg, INPUT_FILE, args, chunks, speeds, TEMP, log)
+                renderAv(ffmpeg, ffprobe, INPUT_FILE, args, chunks, speeds, fps,
+                TEMP, log)
 
             if(args.render == 'opencv'):
                 from renderVideo import renderOpencv
-                renderOpencv(ffmpeg, INPUT_FILE, args, chunks, speeds, fps, TEMP, log)
+                renderOpencv(ffmpeg, ffprobe, INPUT_FILE, args, chunks, speeds, fps,
+                    effects, TEMP, log)
 
             # Now mix new audio(s) and the new video.
             muxVideo(ffmpeg, newOutput, args, tracks, TEMP, log)
@@ -652,7 +673,16 @@ def main():
         smartOpen(newOutput, log)
 
     log.debug('Deleting temp dir')
-    rmtree(TEMP)
+
+    try:
+        rmtree(TEMP)
+    except PermissionError:
+        from time import sleep
+        sleep(1)
+        try:
+            rmtree(TEMP)
+        except PermissionError:
+            log.debug('Failed to delete temp dir.')
 
 if(__name__ == '__main__'):
     main()
