@@ -225,27 +225,44 @@ def setRange(includeFrame: np.ndarray, syntaxRange, fps: float, with_: bool, log
     return includeFrame
 
 
+def secToFrames(value, fps):
+    if(isinstance(value, str)):
+        return int(float(value) * fps)
+    return value
+
+def cook(hasLoud: np.ndarray, minClip: int, minCut: int) -> np.ndarray:
+    # Remove small loudness spikes
+    hasLoud = removeSmall(hasLoud, minClip, replace=True, with_=False)
+    # Remove small silences
+    hasLoud = removeSmall(hasLoud, minCut, replace=False, with_=True)
+    return hasLoud
+
+
+# Turn long silent/loud array to formatted chunk list.
+# Example: [True, True, True, False, False] => [[0, 3, 1], [3, 5, 0]]
+def chunkify(includeFrame, arrayLen=None) -> list:
+    if(arrayLen is None):
+        arrayLen = len(includeFrame)
+
+    chunks = []
+    startP = 0
+    for j in range(1, arrayLen):
+        if(includeFrame[j] != includeFrame[j - 1]):
+            chunks.append([startP, j, int(includeFrame[j-1])])
+            startP = j
+    chunks.append([startP, arrayLen, int(includeFrame[j])])
+    return chunks
+
+
 def applySpacingRules(hasLoud: np.ndarray, fps: float, frameMargin: int,
     minClip: int, minCut: int, ignore, cutOut, log):
 
-    def secToFrames(value, fps):
-        if(isinstance(value, str)):
-            return int(float(value) * fps)
-        return value
-
     frameMargin = secToFrames(frameMargin, fps)
     minClip = secToFrames(minClip, fps)
     minCut = secToFrames(minCut, fps)
 
     log.checkType(frameMargin, 'frameMargin', int)
 
-    def cook(hasLoud: np.ndarray, minClip: int, minCut: int) -> np.ndarray:
-        # Remove small loudness spikes
-        hasLoud = removeSmall(hasLoud, minClip, replace=True, with_=False)
-        # Remove small silences
-        hasLoud = removeSmall(hasLoud, minCut, replace=False, with_=True)
-        return hasLoud
-
     hasLoud = cook(hasLoud, minClip, minCut)
 
     arrayLen = len(hasLoud)
@@ -270,13 +287,175 @@ def applySpacingRules(hasLoud: np.ndarray, fps: float, frameMargin: int,
     # Remove small clips/cuts created by applying other rules.
     includeFrame = cook(includeFrame, minClip, minCut)
 
-    # Turn long silent/loud array to formatted chunk list.
-    # Example: [True, True, True, False, False] => [[0, 3, 1], [3, 5, 0]]
-    chunks = []
+    return chunkify(includeFrame, arrayLen)
+
+
+def applyBasicSpacing(hasLoud: np.ndarray, fps: float, minClip: int, minCut: int, log):
+    minClip = secToFrames(minClip, fps)
+    minCut = secToFrames(minCut, fps)
+
+    includeFrame = cook(hasLoud, minClip, minCut)
+    return chunkify(includeFrame)
+
+
+def merge(start_list: np.ndarray, end_list: np.ndarray) -> np.ndarray:
+    merge = np.zeros((len(start_list)), dtype=np.bool_)
+
     startP = 0
-    for j in range(1, arrayLen):
-        if(includeFrame[j] != includeFrame[j - 1]):
-            chunks.append([startP, j, int(includeFrame[j-1])])
-            startP = j
-    chunks.append([startP, arrayLen, int(includeFrame[j])])
-    return chunks
+    for item in start_list:
+        if(item == True):
+            where_list = np.where(end_list[startP:])[0]
+            if(len(where_list) > 0):
+                merge[startP:where_list[0]] = True
+        startP += 1
+    return merge
+
+
+def handleBoolExp(val: str, data, sampleRate, fps, log) -> list:
+    invert = False
+
+    if('>' in val and '<' in val):
+        log.error('Cannot have both ">" and "<" in same expression.')
+    if('>' in val):
+        exp = val.split('>')
+    elif('<' in val):
+        exp = val.split('<')
+        invert = True
+    else:
+        log.error('audio array needs ">" or "<".')
+
+    if(len(exp) != 2):
+        log.error(f'Only one expression supported, not {len(exp)-1}.')
+
+    if(data is None or sampleRate is None):
+        log.error('No audio data found.')
+
+    new_list = audioToHasLoud(data, sampleRate, float(exp[1]), fps, log)
+
+    if(invert):
+        new_list = np.invert(new_list)
+
+    return new_list
+
+
+def applyRects(cmdRects, audioData, sampleRate, fps, log):
+    from usefulFunctions import hex_to_bgr
+
+    rects = []
+    for ms in cmdRects:
+        if(len(ms) < 6):
+            log.error('Too few comma arguments for rectangle option.')
+
+        if(len(ms) > 8):
+            log.error('Too many comma arguments for rectangle option.')
+
+        start, end, x1, y1, x2, y2 = ms[:6]
+
+        color = '#000'
+        thickness = -1
+        if(len(ms) > 6):
+            color = ms[6]
+        if(len(ms) > 7):
+            thickness = int(ms[7])
+        color = hex_to_bgr(color, log)
+
+        # Handle Boolean Expressions. Mostly the same as zoom.
+        start_list, end_list = None, None
+        if(start.startswith('audio')):
+            start_list = handleBoolExp(start, audioData, sampleRate, fps, log)
+
+        if(end.startswith('audio')):
+            if(start_list is None):
+                log.error('The start parameter must also have a boolean expression.')
+            end_list = handleBoolExp(end, audioData, sampleRate, fps, log)
+
+        if(start_list is None):
+            rects.append(['rectangle', start, end, x1, y1, x2, y2, color, thickness])
+
+        elif(end_list is None):
+            # Handle if end is not a boolean expression.
+            indexs = np.where(start_list)[0]
+            if(indexs != []):
+                rects.append(['rectangle', str(indexs[0]), end, x1, y1, x2, y2, color,
+                    thickness])
+        else:
+            chunks = applyBasicSpacing(merge(start_list, end_list), fps, 0, 0, log)
+            for item in chunks:
+                if(item[2] == 1):
+                    rects.append(['rectangle', str(item[0]), str(item[1]), x1, y1, x2, y2,
+                        color, thickness])
+
+            if(rects == []):
+                log.warning('No rectangles applied.')
+            else:
+                log.print(f' {len(rects)} rectangles applied.')
+
+    return rects
+
+
+def applyZooms(cmdZooms, audioData, sampleRate, fps, log):
+    zooms = []
+    for ms in cmdZooms:
+        if(len(ms) < 3):
+            # Add more description later!
+            log.error('Too few comma arguments for zoom option.' \
+                " Make sure you're wrapping the values in quotes.")
+
+        start, end = ms[:2]
+
+        start_zoom = float(ms[2])
+
+        if(len(ms) == 3):
+            end_zoom = start_zoom
+        else:
+            end_zoom = float(ms[3])
+
+        x = 'centerX'
+        y = 'centerY'
+        inter = 'linear'
+        hold = None
+
+        if(len(ms) > 4):
+            x, y = ms[4:6]
+
+        if(len(ms) > 6):
+            inter = ms[6]
+
+        if(len(ms) > 7):
+            hold = ms[7]
+
+        if(len(ms) > 8):
+            log.error('Too many comma arguments for zoom option.')
+
+        start_list, end_list = None, None
+        if(start.startswith('audio')):
+            start_list = handleBoolExp(start, audioData, sampleRate, fps, log)
+
+        if(end.startswith('audio')):
+            if(start_list is None):
+                log.error('The start parameter must also have a boolean expression.')
+            end_list = handleBoolExp(end, audioData, sampleRate, fps, log)
+
+        if(start_list is None):
+            zooms.append(['zoom', start, end, start_zoom, end_zoom, x, y, inter, hold])
+
+        elif(end_list is None):
+            # Handle if end is not a boolean expression.
+            indexs = np.where(start_list)[0]
+            if(indexs != []):
+                zooms.append(['zoom', str(indexs[0]), end, start_zoom, end_zoom, x, y,
+                    inter, hold])
+        else:
+            chunks = applyBasicSpacing(merge(start_list, end_list), fps, 0, 0, log)
+            for item in chunks:
+                if(item[2] == 1):
+                    zooms.append(['zoom', str(item[0]), str(item[1]), start_zoom, end_zoom,
+                        x, y, inter, hold])
+
+            if(zooms == []):
+                log.warning('No zooms applied.')
+            else:
+                log.print(f' {len(zooms)} zooms applied.')
+
+    log.debug(zooms)
+    return zooms
