@@ -5,16 +5,15 @@ To prevent duplicate code being pasted between scripts, common functions should
 put here. No function should modify or create video/audio files on its own.
 """
 
-# Internal libraries
+# Internal Libraries
 import sys
 from platform import system
 from shutil import get_terminal_size
 from time import time, localtime
 
 class Log():
-    def __init__(self, show_debug=False, ffmpeg=False, quiet=False, temp=None):
+    def __init__(self, show_debug=False, quiet=False, temp=None):
         self.is_debug = show_debug
-        self.is_ffmpeg = ffmpeg
         self.quiet = quiet
         self.temp = temp
 
@@ -22,24 +21,28 @@ class Log():
         if(self.is_debug):
             print(message)
 
-    def ffmpeg(self, message):
-        if(self.is_ffmpeg):
-            print(message)
-
     def cleanup(self):
         if(self.temp is None):
             return
 
         from shutil import rmtree
         rmtree(self.temp)
-        self.debug(f'   - Removed Temp Directory.')
+        self.debug('   - Removed Temp Directory.')
+
+    def conwrite(self, message: str):
+        if(not self.quiet):
+            numSpaces = get_terminal_size().columns - len(message) - 3
+            print('  ' + message + ' ' * numSpaces, end='\r', flush=True)
 
     def error(self, message):
+        self.conwrite('')
+        message = message.replace('\t', '    ')
         print('Error!', message, file=sys.stderr)
         self.cleanup()
         sys.exit(1)
 
     def bug(self, message, bug_type='bug report'):
+        self.conwrite('')
         URL = 'https://github.com/WyattBlue/auto-editor/issues/'
         print('Error!', message,
             "\n\nThis is not a normal error.\n This message will only show up if there's",
@@ -56,11 +59,6 @@ class Log():
         if(not self.quiet):
             print(message, end=end)
 
-    def conwrite(self, message: str):
-        if(not self.quiet):
-            numSpaces = get_terminal_size().columns - len(message) - 3
-            print('  ' + message + ' ' * numSpaces, end='\r', flush=True)
-
     def checkType(self, data, name, correct_type):
         if(not isinstance(data, correct_type)):
             badtype = type(data).__name__
@@ -97,19 +95,25 @@ def sep() -> str:
     return '/'
 
 
+def cleanList(x: list, rm_chars: str) -> list:
+    no = str.maketrans('', '', rm_chars)
+    x = [s.translate(no) for s in x]
+    x = [s for s in x if s != '']
+    return x
+
+
 class FFprobe():
-    def __init__(self, dirPath, myFFmpeg: bool, log):
+    def __init__(self, dirPath, myFFmpeg: bool, FFdebug, log):
         from os import path
 
         self.mylog = log
+        self.FFdebug = FFdebug
 
         newF = None
-
-
         if(system() == 'Windows' and not myFFmpeg):
-            newF = path.join(dirPath, '\\win-ffmpeg\\bin\\ffprobe.exe')
+            newF = path.join(dirPath, f'win-ffmpeg{sep()}bin{sep()}ffprobe.exe')
         if(system() == 'Darwin' and not myFFmpeg):
-            newF = path.join(dirPath, 'mac-ffmpeg/bin/ffprobe')
+            newF = path.join(dirPath, f'mac-ffmpeg{sep()}bin{sep()}ffprobe')
 
         if(newF is not None and path.isfile(newF)):
             self.myPath = newF
@@ -119,19 +123,20 @@ class FFprobe():
                 pipeToConsole([self.myPath, '-h'])
             except FileNotFoundError:
                 if(system() == 'Darwin'):
-                    self.log.error('No ffprobe found, download via homebrew or restore' \
+                    self.mylog.error('No ffprobe found, download via homebrew or restore' \
                         ' the included binary.')
                 elif(system() == 'Windows'):
-                    self.log.error('No ffprobe found, download ffprobe with your' \
+                    self.mylog.error('No ffprobe found, download ffprobe with your' \
                         ' favorite package manager (ex chocolatey), or restore the' \
                         ' included binary.')
                 else:
-                    self.log.error('ffprobe must be on PATH. Download ffprobe by running:\n' \
+                    self.mylog.error('ffprobe must be on PATH. Download ffprobe by running:\n' \
                         '  sudo apt-get install libavformat-dev libavfilter-dev libavdevice-dev ffmpeg' \
                         '\nOr something similar depending on your distro.')
 
     def log(self, message):
-        self.mylog.ffmpeg(message)
+        if(self.FFdebug):
+            print(message)
 
     def getPath(self) -> str:
         return self.myPath
@@ -143,15 +148,18 @@ class FFprobe():
         import subprocess
         full_cmd = [self.myPath] + cmd
 
+        if(None in cmd):
+            self.mylog.bug(f'None in cmd. {cmd}')
+
         self.log(full_cmd)
         subprocess.call(full_cmd)
 
     def pipe(self, cmd: list) -> str:
         full_cmd = [self.myPath, '-v', 'error'] + cmd
 
-        self.log(full_cmd)
+        self.mylog.debug(full_cmd)
         output = pipeToConsole(full_cmd)
-        self.log(output)
+        self.mylog.debug(output)
 
         return output
 
@@ -168,37 +176,56 @@ class FFprobe():
     def getAudioDuration(self, file):
         return self._get(file, 'duration', 'a', 0)
 
-    def getFrameRate(self, file):
+    def getFrameRate(self, file) -> float:
+        # r_frame_rate is better than avg_frame_rate for getting constant frame rate.
         output = self.pipe(['-select_streams', 'v', '-show_entries',
-            'stream=avg_frame_rate', '-of', 'compact=p=0:nk=1', file]).strip()
-        nums = output.split('/')
-        return int(nums[0]) / int(nums[1])
+            'stream=r_frame_rate', '-of', 'compact=p=0:nk=1', file]).strip()
+        nums = cleanList(output.split('/'), '\r\t\n')
+
+        try:
+            return int(nums[0]) / int(nums[1])
+        except (ZeroDivisionError, IndexError, ValueError):
+            self.mylog.error(f'getFrameRate had an invalid output: {output}')
 
     def getAudioTracks(self, file):
         output = self.pipe(['-select_streams', 'a', '-show_entries', 'stream=index',
             '-of', 'compact=p=0:nk=1', file]).strip()
 
-        numbers = output.split('\n')
-        # Remove all \r chars that can appear in certain environments
-        numbers = [s.replace('\r', '') for s in numbers]
-        # Remove all blanks
-        numbers = [s for s in numbers if s != '']
-
-        self.log('Track data: ' + str(numbers))
+        numbers = cleanList(output.split('\n'), '\r\t')
+        self.log(f'Track data: {numbers}')
         if(numbers[0].isnumeric()):
             return len(numbers)
         else:
             self.mylog.warning('ffprobe had an invalid output.')
             return 1 # Assume there's one audio track.
 
+    def getSubtitleTracks(self, file):
+        output = self.pipe(['-select_streams', 's', '-show_entries', 'stream=index',
+            '-of', 'compact=p=0:nk=1', file]).strip()
+
+        numbers = cleanList(output.split('\n'), '\r\t')
+        self.log(f'Track data: {numbers}')
+        if(numbers[0].isnumeric()):
+            return len(numbers)
+        else:
+            self.mylog.warning('Invalid output when detecting number of subtitle tracks.')
+            return 0
+
+    def getLang(self, file, track=0):
+        return self.pipe(['-select_streams', f's:{track}', '-show_entries',
+            'stream_tags=language', '-of', 'csv=p=0', file])
+
     def getAudioCodec(self, file, track=0):
         return self._get(file, 'codec_name', 'a', track)
 
+    def getVideoCodec(self, file, track=0):
+        return self._get(file, 'codec_name', 'v', track)
+
     def getSampleRate(self, file, track=0):
         return self._get(file, 'sample_rate', 'a', track)
 
-    def getAudioBitrate(self, file, track=0):
-        return self._get(file, 'bit_rate', 'a', track)
+    def getBitrate(self, file, the_type='v', track=0):
+        return self._get(file, 'bit_rate', the_type, track)
 
     def getPrettySampleRate(self, file, track=0) -> str:
         output = self.getSampleRate(file, track)
@@ -206,25 +233,26 @@ class FFprobe():
             return str(int(output) / 1000) + ' kHz'
         return 'N/A'
 
-    def getPrettyABitrate(self, file, track=0) -> str:
-        output = self.getAudioBitrate(file, track)
+    def getPrettyBitrate(self, file, the_type='v', track=0) -> str:
+        output = self.getBitrate(file, the_type, track)
         if(output.isnumeric()):
             # This does get used by ffmpeg so be careful.
             return str(round(int(output) / 1000)) + 'k'
         return 'N/A'
 
 class FFmpeg():
-    def __init__(self, dirPath, myFFmpeg: bool, log):
+    def __init__(self, dirPath, myFFmpeg: bool, FFdebug, log):
 
         from os import path
 
         self.mylog = log
+        self.FFdebug = FFdebug
 
         newF = None
         if(system() == 'Windows' and not myFFmpeg):
-            newF = path.join(dirPath, '\\win-ffmpeg\\bin\\ffmpeg.exe')
+            newF = path.join(dirPath, f'win-ffmpeg{sep()}bin{sep()}ffmpeg.exe')
         if(system() == 'Darwin' and not myFFmpeg):
-            newF = path.join(dirPath, 'mac-ffmpeg/bin/ffmpeg')
+            newF = path.join(dirPath, f'mac-ffmpeg{sep()}bin{sep()}ffmpeg')
 
         if(newF is not None and path.isfile(newF)):
             self.myPath = newF
@@ -234,19 +262,20 @@ class FFmpeg():
                 pipeToConsole([self.myPath, '-h'])
             except FileNotFoundError:
                 if(system() == 'Darwin'):
-                    self.log.error('No ffmpeg found, download via homebrew or restore' \
+                    self.mylog.error('No ffmpeg found, download via homebrew or restore' \
                         ' the included binaries.')
                 elif(system() == 'Windows'):
-                    self.log.error('No ffmpeg found, download ffmpeg with your' \
+                    self.mylog.error('No ffmpeg found, download ffmpeg with your' \
                         ' favorite package manager (ex chocolatey), or restore the' \
                         ' included binaries.')
                 else:
-                    self.log.error('FFmpeg must be on PATH. Download ffmpeg by running:\n' \
+                    self.mylog.error('FFmpeg must be on PATH. Download ffmpeg by running:\n' \
                         '  sudo apt-get install libavformat-dev libavfilter-dev libavdevice-dev ffmpeg' \
                         '\nOr something similar depending on your distro.')
 
     def log(self, message):
-        self.mylog.ffmpeg(message)
+        if(self.FFdebug):
+            print(message)
 
     def getPath(self) -> str:
         return self.myPath
@@ -256,11 +285,11 @@ class FFmpeg():
 
     def run(self, cmd: list):
         cmd = [self.myPath, '-y'] + cmd
-        if(self.mylog.is_ffmpeg):
+        if(self.FFdebug):
             cmd.extend(['-hide_banner'])
         else:
             cmd.extend(['-nostats', '-loglevel', '8'])
-        self.log(cmd)
+        self.mylog.debug(cmd)
 
         import subprocess
         subprocess.call(cmd)
@@ -268,9 +297,9 @@ class FFmpeg():
     def pipe(self, cmd: list) -> str:
         cmd = [self.myPath, '-y'] + cmd
 
-        self.log(cmd)
+        self.mylog.debug(cmd)
         output = pipeToConsole(cmd)
-        self.log(output)
+        self.mylog.debug(output)
 
         return output
 
@@ -379,16 +408,18 @@ class ProgressBar():
             print(f'   {percentDone}% done ETA {newTime}')
 
 
-def isLatestVersion(version, log) -> bool:
-    try:
-        from requests import get
-        latestVersion = get('https://raw.githubusercontent.com/' \
-            'wyattblue/auto-editor/master/resources/version.txt')
-        return latestVersion == version
-    except ImportError:
-        pass
-    except Exception as err:
-        log.debug('Connection Error: ' + str(err))
+def isLatestVersion(version: str, log) -> bool:
+    if('dev' not in version):
+        try:
+            from requests import get
+            latestVersion = get('https://raw.githubusercontent.com/' \
+                'wyattblue/auto-editor/master/resources/version.txt')
+            return latestVersion.text == version
+        except ImportError:
+            pass
+        except Exception as err:
+            log.debug('Connection Error: ' + str(err))
+    return True
 
 def humanReadableTime(rawTime: float) -> str:
     units = 'seconds'
@@ -416,4 +447,16 @@ def smartOpen(newOutput: str, log):
             try: # should work on WSL2
                 call(['cmd.exe', '/C', 'start', newOutput])
             except:
-                log.warning('Could not open output file.')
+                try: # should work on various other Linux distros
+                    call(['xdg-open', newOutput])
+                except:
+                    log.warning('Could not open output file.')
+
+def hex_to_bgr(inp: str, log) -> list:
+    import re
+    if(re.compile(r'#[a-fA-F0-9]{3}(?:[a-fA-F0-9]{3})?$').match(inp)):
+        if(len(inp) < 5):
+            return [int(inp[i]*2, 16) for i in (3, 2, 1)]
+        return [int(inp[i:i+2], 16) for i in (5, 3, 1)]
+    else:
+        log.error(f'Invalid hex code: {inp}')
