@@ -1,15 +1,19 @@
 '''vanparse.py'''
 
-import difflib
+import os
 import sys
+import difflib
 
-def printHelp(option, args):
-    print(' ', ', '.join(option['names']))
+def printHelp(args, option):
     if(option['action'] == 'grouping'):
-        print('   |')
+        print(f'  {option["names"][0]}:')
     else:
+        print(' ', ', '.join(option['names']))
         print('   ', option['help'])
-        print('   ', option['extra'])
+        if(option['extra'] == ''):
+            print('')
+        else:
+            print(f"\n{option['extra']}\n")
     if(option['action'] == 'default'):
         print('    type:', option['type'].__name__)
         print('    default:', option['default'])
@@ -22,16 +26,16 @@ def printHelp(option, args):
             for op in options:
                 if(op['grouping'] == option['names'][0]):
                     print(' ', ', '.join(op['names']) + ':', op['help'])
-                    if(op['action'] == 'grouping'):
-                        print('     ...')
-
     elif(option['action'] == 'store_true'):
         print('    type: flag')
     else:
         print('    type: unknown')
 
+    if(option['grouping'] is not None):
+        print(f'    group: {option["grouping"]}')
 
-def get_option(item, group, the_args: list):
+
+def get_option(item: str, group: str, the_args: list) -> str:
     for options in the_args:
         for option in options:
             if(item in option['names']):
@@ -57,15 +61,12 @@ class ParseOptions():
                     value = option['default']
                 setattr(self, key, value)
 
-
-        # parse config file
-        import os
-
+        # Read the configuration file.
         dirPath = os.path.dirname(os.path.realpath(__file__))
-
         with open(dirPath + '/config.txt', 'r') as file:
             lines = file.readlines()
 
+        # Set attributes based on the config file to act as the new defaults.
         for item in lines:
             if('#' in item):
                 item = item[: item.index('#')]
@@ -84,72 +85,81 @@ class ParseOptions():
                 value = int(value)
 
             key = item[: item.index('=')]
-
             key = key[key.rfind('.')+1:]
-
             setattr(self, key, value)
 
-
-        # Figure out attributes changed by user.
+        # Figure out command line options changed by user.
         myList = []
+        used_options = []
         settingInputs = True
         optionList = 'input'
+        listType = str
         i = 0
         group = None
         while i < len(userArgs):
             item = userArgs[i]
-            if(i == len(userArgs) - 1):
-                nextItem = None
+            label = 'option' if item.startswith('--') else 'short'
+
+            # Find the option.
+            if(label == 'option'):
+                option = get_option(item, group='global', the_args=args)
             else:
-                nextItem = userArgs[i+1]
-
-            option = get_option(item, group, args)
-
-            if(option is None and (group is not None)):
-                group = None
-                option = get_option(item, group, args)
+                option = get_option(item, group=group, the_args=args)
+                if(option is None and (group is not None)):
+                    group = None # Don't consider the next option to be in a group.
+                    option = get_option(item, group=group, the_args=args)
 
             if(option is None):
+                # Unknown Option!
                 if(settingInputs and not item.startswith('-')):
-                    # Input file names
+                    # Option is actually an input file, like example.mp4
                     myList.append(item)
                 else:
-                    # Unknown Option!
-                    hmm = difflib.get_close_matches(item, option_names)
-                    append = ''
+                    # Get the names of all the options and groups.
+                    opt_list = []
+                    for options in args:
+                        for opt in options:
+                            for names in opt['names']:
+                                opt_list.append(names)
 
-                    # If there's an exact match.
-                    if(len(hmm) > 0 and hmm[0] == item):
-                        option = get_option(item, 'global', args)
-                        group = option['grouping']
-                        myDefault = option['default'] if option['action'] != 'store_true' else ''
-                        append = f'\n\nExample:\n    auto-editor {group} {item} {myDefault}'
-                        log.error(f'Option {item} needs to be in group: {group}{append}')
-
-                    potential_options = ', '.join(hmm)
-
-                    if(hmm != []):
-                        append = f'\n\n    Did you mean:\n        {potential_options}'
-                    log.error(f'Unknown option: {item}{append}')
+                    close_matches = difflib.get_close_matches(item, opt_list)
+                    # Throw an error of some kind.
+                    if(close_matches):
+                        if(close_matches[0] == item):
+                            # Option exists but is not in the right group.
+                            log.error(f'{label.capitalize()} {item} needs to be in a group')
+                        else:
+                            log.error(f'Unknown {label}: {item}\n\n\t' \
+                                'Did you mean:\n\t\t' + ', '.join(close_matches))
+                    else:
+                        log.error(f'Unknown {label}: {item}')
             else:
+                # We found the option.
                 if(optionList is not None):
-                    setattr(self, optionList, myList)
+                    setattr(self, optionList, list(map(listType, myList)))
+
                 settingInputs = False
                 optionList = None
                 myList = []
 
-                key = option['names'][0].replace('-', '')
+                if(option['names'][0] in used_options and option['stack'] is None):
+                    log.error(f'Cannot repeat option {option["names"][0]} twice.')
 
+                used_options.append(option['names'][0])
+
+                key = option['names'][0].replace('-', '')
                 if(option['action'] == 'grouping'):
                     group = key
 
+                nextItem = None if i == len(userArgs) - 1 else userArgs[i+1]
                 if(nextItem == '-h' or nextItem == '--help'):
-                    printHelp(option, args)
+                    printHelp(args, option)
                     sys.exit()
 
                 if(option['nargs'] != 1):
                     settingInputs = True
                     optionList = key
+                    listType = option['type']
                 elif(option['action'] == 'store_true'):
                     value = True
                 else:
@@ -160,15 +170,32 @@ class ParseOptions():
                         typeName = option['type'].__name__
                         log.debug(f'Exact Error: {err}')
                         log.error(f'Couldn\'t convert "{nextItem}" to {typeName}')
-                    if(option['choices'] is not None):
-                        if(value not in option['choices']):
-                            optionName = option['names'][0]
-                            myChoices = ', '.join(option['choices'])
-                            log.error(f'{value} is not a choice for {optionName}' \
-                                f'\nchoices are:\n  {myChoices}')
+
+                    # Handle when the option value is not in choices list.
+                    if(option['choices'] is not None and value not in option['choices']):
+                        optionName = option['names'][0]
+                        myChoices = ', '.join(option['choices'])
+                        log.error(f'{value} is not a choice for {optionName}' \
+                            f'\nchoices are:\n  {myChoices}')
                     i += 1
                 setattr(self, key, value)
 
             i += 1
         if(settingInputs):
-            setattr(self, optionList, myList)
+            setattr(self, optionList, list(map(listType, myList)))
+
+        if(self.help):
+            for options in args:
+                for op in options:
+                    if(op['action'] == 'grouping'):
+                        print(f"\n  {op['names'][0]}:")
+                    else:
+                        print(' ', ', '.join(op['names']) + ':', op['help'])
+            print('')
+            if(root == 'auto-editor'):
+                print('  Have an issue? Make an issue. '\
+                    'Visit https://github.com/wyattblue/auto-editor/issues\n')
+                print('  The help option can also be used on a specific option:')
+                print('      auto-editor --frame_margin --help\n')
+            sys.exit()
+
